from collections import namedtuple
from typing import Mapping, List

Domain = namedtuple('Domain', 'domain keys')


class PayloadContent:
    @staticmethod
    def create(plist: Mapping) -> 'PayloadContent':
        return {
            'com.apple.system.logging': LoggingPayload,
            'com.apple.defaults.managed': ManagedPayload,
            'com.apple.corecapture.configure': CoreCapture,
            'com.apple.ManagedClient.preferences': ManagedClient
        }[plist['PayloadType']](plist)

    def __init__(self, plist: Mapping):
        self._plist = plist

    @property
    def payload_content(self) -> Mapping:
        return self._plist['PayloadContent']

    @property
    def payload_type(self) -> str:
        return self._plist['PayloadType']


class LoggingPayload(PayloadContent):
    def __repr__(self):
        subsystems = self._plist.get('Subsystems')
        description = ''
        if subsystems:
            description = 'Subsystems: '
            description += ', '.join(subsystems.keys())
        return description


class ManagedPayload(PayloadContent):
    @property
    def domains(self) -> List[Domain]:
        result = []
        for inner_payload_content in self.payload_content:
            domain = inner_payload_content.get('DefaultsDomainName')
            keys = inner_payload_content.get('DefaultsData').keys()
            result.append(Domain(domain, keys))
        return result

    def __repr__(self):
        description = ''
        for inner_payload_content in self.payload_content:
            domain = inner_payload_content.get('DefaultsDomainName')
            keys = ', '.join(inner_payload_content.get('DefaultsData').keys())
            description += f'{domain}: {keys}'
        return description


class CoreCapture(PayloadContent):
    @property
    def core_capture_config(self) -> List[str]:
        """ a special property present only in payloads of type com.apple.ManagedClient.preferences """
        return self._plist['CoreCaptureConfig'].keys()

    def __repr__(self):
        return str(self.core_capture_config)


class ManagedClient(PayloadContent):
    def __repr__(self):
        return str(self.payload_content)


class MobileConfig:
    def __init__(self, plist: Mapping):
        self._plist = plist

    @property
    def payload_display_name(self) -> str:
        return self._plist['PayloadDisplayName']

    @property
    def payload_content(self) -> List[PayloadContent]:
        return [PayloadContent.create(p) for p in self._plist['PayloadContent']]



import csv
import plistlib
from pathlib import Path
from typing import Mapping

import click
import mdutils

from mobileconfig.mobileconfig import MobileConfig, ManagedPayload

XML_HEADER = b'<?xml '
XML_TAIL = b'</plist>'


def get_plist(file: bytes) -> Mapping:
    return plistlib.loads(XML_HEADER + file.split(XML_HEADER, 1)[1].split(XML_TAIL, 1)[0] + XML_TAIL)


@click.group()
@click.argument('directory', type=click.Path(dir_okay=True, file_okay=False, exists=True))
@click.pass_context
def cli(ctx, directory):
    """ Parse .mobileconfig files in a given directory """
    ctx.ensure_object(dict)
    ctx.obj['directory'] = directory
    ctx.obj['plists'] = []
    for file in Path(directory).glob('*.mobileconfig'):
        ctx.obj['plists'].append(get_plist(file.read_bytes()))


@cli.command()
@click.pass_context
def consents(ctx):
    """ Print all user consents """
    for plist in ctx.obj['plists']:
        consent_text = plist['ConsentText']
        consent_text_en = consent_text.get('default', consent_text.get('en'))
        print(consent_text_en)


@cli.command()
@click.argument('output_file', type=click.File('w'), required=True)
@click.argument('output_type', type=click.Choice(['csv', 'md']), required=True)
@click.pass_context
def payload_types(ctx, output_file, output_type):
    """ Print all PayloadTypes to either a csv or md file """
    if output_type == 'csv':
        output_file = csv.writer(output_file)
    if output_type == 'md':
        mdfile = mdutils.MdUtils(file_name=output_file.name, title='Profiles')


    rows = [('Display Name', 'PayloadType', 'Description')]
    for plist in ctx.obj['plists']:
        mobileconfig = MobileConfig(plist)
        display_name = mobileconfig.payload_display_name
        for payload_content in mobileconfig.payload_content:
            payload_type = payload_content.payload_type

            if isinstance(payload_content, ManagedPayload):
                for domain in payload_content.domains:
                    rows.append((display_name, payload_type, f'{domain.domain}: {", ".join(domain.keys)}'))
            else:
                description = str(payload_content)
                rows.append((display_name, payload_type, description))

    if output_type == 'csv':
        output_file.writerows(rows)

    if output_type == 'md':
        mdfile.new_table(columns=3, rows=len(rows) // 3, text=rows, text_align='left')
        mdfile.create_md_file()


@cli.command()
@click.argument('output', type=click.Path(dir_okay=True, file_okay=False, exists=False))
@click.pass_context
def extract(ctx, output):
    """ Extract the plists into given directory """
    output = Path(output)
    output.mkdir(0o777, exist_ok=True, parents=True)
    for plist in ctx.obj['plists']:
        display_name = plist["PayloadDisplayName"]
        filename = display_name.replace('/', '_') + '.plist'
        (output / filename).write_bytes(plistlib.dumps(plist))


if __name__ == '__main__':
    cli()
